﻿using System;
using System.Collections.Generic;
using System.Web;
using WebMatrix.Data;
using System.Configuration;
using System.Linq;

//Gestor de la base de datos utilizado por el gestor
// de pantallas para acceder a los datos
namespace GestorBBDD
{
    //Conexión con la base de datos
    public static class Conexion
    {
        private const string NOMBRE_BBDD = "tiendaConnectionString";
        public static Database db;
        
        public static void Establecer()
        {
            db = Database.Open(NOMBRE_BBDD);
        }
        
        public static void Cerrar()
        {
            db.Close();
        }
        
        public static string ObtenerCadena()
        {
            return ConfigurationManager.ConnectionStrings[NOMBRE_BBDD].ConnectionString;
        }
        
        public static Database Obtener()
        {
            return db;
        }
    }
    
    //Clases de las diferentes entidades con sus respectivos métodos
    public static class Usuarios
    {
        public static int? Alta(Usuario usuario, Domicilio domicilio)
        {
            Object resultado;
            int? idUsuario;
            
            resultado = Gestion.EjecutarProcedimiento("dbo.sp_AltaUsuario",
                usuario.nombreUsuario, usuario.contrasena, usuario.nombre, usuario.apellidos,
                usuario.fechaNacimiento, usuario.telefonoMovil, usuario.correoElectronico,
                domicilio.tipoVia, domicilio.direccion, domicilio.codigoPostal, domicilio.pais,
                domicilio.provincia, domicilio.localidad, domicilio.telefonoFijo);
            
            idUsuario = resultado == null ? (int?)null : Convert.ToInt32((decimal)resultado);
            
            return idUsuario;
        }
        
        public static bool Autenticar(string nombreUsuario, string contrasena, out bool esGestor)
        {
            bool resultado;
            
            resultado = Gestion.ConsultarFuncionEscalar("dbo.f_Identificacion", nombreUsuario, contrasena);
            
            if(resultado)
            {
                esGestor = Gestion.ConsultarFuncionEscalar("dbo.f_esGestor", nombreUsuario);
            }
            else
            {
                esGestor = false;
            }
            
            return resultado;
        }
        
        public static bool Existe(string nombreUsuario)
        {
            return Gestion.ConsultarFuncionEscalar("dbo.f_existeUsuario", nombreUsuario);
        }
        
        public static int? ObtenerID(string nombreUsuario)
        {
            return Gestion.ConsultarFuncionEscalar("dbo.f_ObtenerIDUsuario", nombreUsuario);
        }
    }
    
    public static class Domicilios
    {
        public static int? Alta(Domicilio domicilio)
        {       
            Object resultado;
            int? idDomicilio;
            
           resultado = Gestion.EjecutarProcedimiento("dbo.sp_AltaDomicilio",
                domicilio.idUsuario, domicilio.tipoVia, domicilio.direccion, domicilio.codigoPostal, domicilio.pais,
                domicilio.provincia, domicilio.localidad, domicilio.telefonoFijo);
            
            idDomicilio = resultado == null ? (int?)null : Convert.ToInt32((decimal)resultado);
            
            return idDomicilio;
        }
    }
    
    public static class Categorias
    {
        public static dynamic ConsultarMenu()
        {
            return Gestion.ConsultarVista("dbo.vw_Portada_Categorias ORDER BY id_cat_superior, nivel, descripcion");
        }
        
        public static dynamic ConsultarDescripcion()
        {            
            return Gestion.ConsultarVista("dbo.vw_Descripcion_Categorias ORDER BY id_cat_superior, nivel, descripcion");
        }
        
        public static int? Alta(Categoria categoria)
        {
            Object resultado;
            int? idCategoria;
            
            resultado = Gestion.EjecutarProcedimiento("dbo.sp_AdmCategorias",
                categoria.idCatSuperior, categoria.nivel, categoria.descripcion);
            
            idCategoria = resultado == null ? (int?)null : Convert.ToInt32((decimal)resultado);
            
            return idCategoria;
        }
    }
    
    public static class Productos
    {
        public static dynamic ConsultarMenu(int? idCategoria)
        {
            dynamic resultado;
            
            if(idCategoria != null)
            {
                resultado = Gestion.ConsultarFuncionTabla("dbo.f_Portada_Productos", idCategoria);
            }
            else
            {
                resultado = Gestion.ConsultarVista("dbo.vw_Portada_Productos ORDER BY id DESC");
            }
            return resultado;
        }
        
        public static dynamic ConsultarDescripcion()
        {
            return Gestion.ConsultarVista("dbo.vw_Descripcion_Productos ORDER BY descripcion");
        }
        
        public static dynamic Consultar(int idProducto)
        {
            return Gestion.ConsultarFuncionTabla("dbo.f_DetalleProducto", idProducto);
        }
        
        public static dynamic Buscar(string literalBuscado)
        {
            return Gestion.ConsultarFuncionTabla("dbo.f_BusquedaProductos", literalBuscado);
        }
        
        public static void Suscribirse(int idUsuario, int idProducto)
        {
            Gestion.EjecutarProcedimiento("dbo.sp_DetalleProducto_Suscripcion" , idUsuario, idProducto);
        }
        
        public static int? Alta(Producto producto)
        {
            Object resultado;
            int? idProducto;
            
            resultado = Gestion.EjecutarProcedimiento("dbo.sp_AdmProductos",
                producto.idCategoria, producto.precio, producto.descripcionBreve,
                producto.descripcionLarga, producto.rutaFoto);
            
            idProducto = resultado == null ? (int?)null : Convert.ToInt32((decimal)resultado);
            
            return idProducto;
        }
    }
    
    public static class Ofertas
    {
        public static dynamic ConsultarMenu(int? idCategoria)
        {
            dynamic resultado;
            
            if(idCategoria != null)
            {
                resultado = Gestion.ConsultarFuncionTabla("dbo.f_Portada_Ofertas", idCategoria);
            }
            else
            {
                resultado = Gestion.ConsultarVista("dbo.vw_Portada_Ofertas ORDER BY id DESC");
            }
            
            return resultado;
        }
        
        public static int? Alta(Oferta oferta)
        {
            Object resultado;
            int? idOferta;
            
            resultado = Gestion.EjecutarProcedimiento("dbo.sp_AdmOfertas",
                oferta.idProducto, oferta.descripcion, oferta.fechaInicio,
                oferta.fechaFin, oferta.tipoOferta);
            
            idOferta = resultado == null ? (int?)null : Convert.ToInt32((decimal)resultado);
            
            return idOferta;
        }
    }

    public static class TipoOfertas
    {
        public static dynamic ConsultarDescripcion()
        {
            return Gestion.ConsultarVista("dbo.vw_Descripcion_TipoOfertas ORDER BY descripcion");
        }
    }
    
    public static class Gestores
    {
        public static dynamic ConsultarDescripcion()
        {
            return Gestion.ConsultarVista("dbo.vw_Descripcion_Gestores ORDER BY descripcion");
        }
        
        public static int? Alta(Gestor gestor)
        {
            Object resultado;
            int? idGestor;
            
            resultado = Gestion.EjecutarProcedimiento("dbo.sp_AdmGestores",
                gestor.idUsuario, gestor.fechaInicio, gestor.fechaFin,
                gestor.permGestores, gestor.permProductos,
                gestor.permCategorias, gestor.permOfertas);
            
            idGestor = resultado == null ? (int?)null : Convert.ToInt32((decimal)resultado);
            
            return idGestor;
        }
    }
    
    public static class ListasProductos
    {
        public static void Alta(ListaProductos listaProductos)
        {
            Gestion.EjecutarProcedimiento("dbo.sp_Cesta_Guardar",
                listaProductos.idUsuario, listaProductos.idProducto,
                listaProductos.cantidad);    
        }
        
        public static void Comprar(ListaProductos listaProductos)
        {
            Gestion.EjecutarProcedimiento("dbo.sp_Cesta_Comprar",
                listaProductos.idUsuario, listaProductos.idProducto);    
        }
        
        public static dynamic Consultar(int idUsuario)
        {
            return Gestion.ConsultarFuncionTabla("dbo.f_Cesta_ListasProductos", idUsuario);
        }
    }
    
    public static class TipoVias
    {
        public static dynamic ConsultarDescripcion()
        {
            return Gestion.ConsultarVista("dbo.vw_Descripcion_TipoVias ORDER BY descripcion");
        }
    }
    
    //Clase para el manejo de los elementos de la base de datos: procedimientos,
    //vistas, funciones escalares y funciones de tabla
    public static class Gestion
    {
        public static Object EjecutarProcedimiento(string procedimiento, params Object[] args)
        {
            var parametros = args.Select((s, i) => "@" + i.ToString()).ToArray();
            
            string cadena = string.Format("EXECUTE {0} {1}", procedimiento, string.Join(",", parametros));
            
            int indice;
            
            for(indice = 0; indice < args.Count(); indice++)
            {
                if (args[indice] == null)
                {
                    args[indice] = DBNull.Value;
                }
            }
            
            return Conexion.Obtener().QueryValue(cadena, args);
        }
        
        public static dynamic ConsultarVista(string vista)
        {
            string cadena = string.Format("SELECT * FROM {0}", vista);
            
            return Conexion.Obtener().Query(cadena);   
        }
        
        public static dynamic ConsultarFuncionTabla(string funcion, params Object[] args)
        {
            var parametros = args.Select((s, i) => "@" + i.ToString()).ToArray();
            
            string cadena = string.Format("SELECT * FROM {0} ({1})", funcion, string.Join(",", parametros));
            
            return Conexion.Obtener().Query(cadena, args);
        }
        
        public static dynamic ConsultarFuncionEscalar(string funcion, params Object[] args)
        {
            var parametros = args.Select((s, i) => "@" + i.ToString()).ToArray();
            
            string cadena = string.Format("SELECT {0} ({1})", funcion, string.Join(",", parametros));
            
            return Conexion.Obtener().QueryValue(cadena, args);
        }
    }
}