﻿using System;
using System.Collections.Generic;
using System.Web;
using WebMatrix.Data;
using System.Configuration;
using System.Linq;

//Gestor de pantallas utilizado desde éstas para acceder a los datos
namespace GestorPantallas
{
    public static class AltaUsuario
    {
        public static void Cargar(out dynamic tipoVias)
        {
            GestorBBDD.Conexion.Establecer();
                
            tipoVias = GestorBBDD.TipoVias.ConsultarDescripcion();
                
            GestorBBDD.Conexion.Cerrar();
        }
        
        public static int? Guardar(Usuario usuario, Domicilio domicilio)
        {
            int? idUsuario;
            
            GestorBBDD.Conexion.Establecer();
            
            idUsuario = GestorBBDD.Usuarios.Alta(usuario, domicilio);
            
            GestorBBDD.Conexion.Cerrar();
            
            return idUsuario;
        }
        
        public static bool Existe(string nombreUsuario)
        {
            bool resultado;
            
            GestorBBDD.Conexion.Establecer();
                
            resultado = GestorBBDD.Usuarios.Existe(nombreUsuario);
                
            GestorBBDD.Conexion.Cerrar();
            
            return resultado;
        }
    }
    
    public static class AltaDomicilio
    {
        public static void Cargar(out dynamic tipoVias)
        {
            GestorBBDD.Conexion.Establecer();
                
            tipoVias = GestorBBDD.TipoVias.ConsultarDescripcion();
                
            GestorBBDD.Conexion.Cerrar();
        }
        
        public static int? Guardar(Domicilio domicilio)
        {
            int? idDomicilio;
            
            GestorBBDD.Conexion.Establecer();
            
            idDomicilio = GestorBBDD.Domicilios.Alta(domicilio);
            
            GestorBBDD.Conexion.Cerrar();
            
            return idDomicilio;
        }
    }
    
    public static class Portada
    {
        public static void Cargar(out dynamic categorias, out dynamic ofertas, out dynamic productos, int? idCategoria)
        {
            GestorBBDD.Conexion.Establecer();
                
            categorias = GestorBBDD.Categorias.ConsultarMenu();
            ofertas = GestorBBDD.Ofertas.ConsultarMenu(idCategoria);
            productos = GestorBBDD.Productos.ConsultarMenu(idCategoria);
                
            GestorBBDD.Conexion.Cerrar();
        }
    }
    
    public static class BusquedaProductos
    {
        public static void Cargar(string literalBuscado, out dynamic productos)
        {
            GestorBBDD.Conexion.Establecer();
                
            productos = GestorBBDD.Productos.Buscar(literalBuscado);
                
            GestorBBDD.Conexion.Cerrar();
        } 
    }
    
    public static class Identificacion
    {
        public static bool Comprobar(string nombreUsuario, string contrasena, out bool esGestor)
        {
            bool resultado;
            
            GestorBBDD.Conexion.Establecer();
                
            resultado = GestorBBDD.Usuarios.Autenticar(nombreUsuario, contrasena, out esGestor);
                
            GestorBBDD.Conexion.Cerrar();
            
            return resultado;
        }
        
        public static bool Existe(string nombreUsuario)
        {
            bool resultado;
            
            GestorBBDD.Conexion.Establecer();
                
            resultado = GestorBBDD.Usuarios.Existe(nombreUsuario);
                
            GestorBBDD.Conexion.Cerrar();
            
            return resultado;
        }
        
        public static int? ObtenerID(string nombreUsuario)
        {
            int? resultado;
            
            GestorBBDD.Conexion.Establecer();
                
            resultado = GestorBBDD.Usuarios.ObtenerID(nombreUsuario);
                
            GestorBBDD.Conexion.Cerrar();
            
            return resultado;
        }
    }
    
    public static class DetalleProducto
    {
        public static void Cargar(int idProducto, out dynamic producto)
        {
            GestorBBDD.Conexion.Establecer();
                
            producto = GestorBBDD.Productos.Consultar(idProducto);
                
            GestorBBDD.Conexion.Cerrar();
        }
        
        public static void Suscribirse(int idUsuario, int idProducto)
        {
            GestorBBDD.Conexion.Establecer();
                
            GestorBBDD.Productos.Suscribirse(idUsuario, idProducto);
                
            GestorBBDD.Conexion.Cerrar();
        }
    }
    
    public static class AdmProductos
    {
        public static int? Guardar(Producto producto)
        {
            int? idProducto;
            
            GestorBBDD.Conexion.Establecer();
            
            idProducto = GestorBBDD.Productos.Alta(producto);
            
            GestorBBDD.Conexion.Cerrar();
            
            return idProducto;
        }
        
        public static void Cargar(out dynamic categorias)
        {
            GestorBBDD.Conexion.Establecer();
                
            categorias = GestorBBDD.Categorias.ConsultarDescripcion();
                
            GestorBBDD.Conexion.Cerrar();
        }
    }
    
    public static class AdmOfertas
    {
        public static int? Guardar(Oferta oferta)
        {
            int? idOferta;
            
            GestorBBDD.Conexion.Establecer();
            
            idOferta = GestorBBDD.Ofertas.Alta(oferta);
            
            GestorBBDD.Conexion.Cerrar();
            
            return idOferta;
        }
        
        public static void Cargar(out dynamic productos, out dynamic tipoOfertas)
        {
            GestorBBDD.Conexion.Establecer();
                
            productos = GestorBBDD.Productos.ConsultarDescripcion();
            tipoOfertas = GestorBBDD.TipoOfertas.ConsultarDescripcion();
                
            GestorBBDD.Conexion.Cerrar();
        }
    }
    
    public static class AdmGestores
    {
        public static int? Guardar(Gestor gestor)
        {
            int? idGestor;
            
            GestorBBDD.Conexion.Establecer();
            
            idGestor = GestorBBDD.Gestores.Alta(gestor);
            
            GestorBBDD.Conexion.Cerrar();
            
            return idGestor;
        }
        
        public static void Cargar(out dynamic gestores)
        {
            GestorBBDD.Conexion.Establecer();
                
            gestores = GestorBBDD.Gestores.ConsultarDescripcion();
                
            GestorBBDD.Conexion.Cerrar();
        }
    }
    
    public static class AdmCategorias
    {
        public static int? Guardar(Categoria categoria)
        {
            int? idCategoria;
            
            GestorBBDD.Conexion.Establecer();
            
            idCategoria = GestorBBDD.Categorias.Alta(categoria);
            
            GestorBBDD.Conexion.Cerrar();
            
            return idCategoria;
        }
        
        public static void Cargar(out dynamic categorias)
        {
            GestorBBDD.Conexion.Establecer();

            categorias = GestorBBDD.Categorias.ConsultarDescripcion();
                
            GestorBBDD.Conexion.Cerrar();
        }
    }
    
    public static class Cesta
    {
        public static void Guardar(dynamic listasProductos)
        {
            GestorBBDD.Conexion.Establecer();
            
            foreach(ListaProductos producto in listasProductos)
            {
                GestorBBDD.ListasProductos.Alta(producto);
            }
            
            GestorBBDD.Conexion.Cerrar();
        }
        
        public static void Comprar(dynamic listasProductos)
        {
            GestorBBDD.Conexion.Establecer();
            
            foreach(ListaProductos producto in listasProductos)
            {
                GestorBBDD.ListasProductos.Comprar(producto);
            }
            
            GestorBBDD.Conexion.Cerrar();
        }
        
        public static void Cargar(int idUsuario, out dynamic listasProductos)
        {
            GestorBBDD.Conexion.Establecer();

            listasProductos = GestorBBDD.ListasProductos.Consultar(idUsuario);
                
            GestorBBDD.Conexion.Cerrar();
        }
    }
}