﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Globalization;
using System.Text.RegularExpressions;

namespace Utilidades
{
    public static class Fechas
    {
        public const string FORMATO_FECHA = "dd/MM/yyyy";
        
        public static DateTime ObtenerFecha(string valor, string formato)
        {
            return DateTime.ParseExact(valor, formato, CultureInfo.InvariantCulture);
        }
     
        public static bool EsValida(string valor, string formato)
        {
            DateTime resultado;
            
            return DateTime.TryParseExact(valor, formato, CultureInfo.InvariantCulture, DateTimeStyles.None, out resultado);
        }
        
        public static string ObtenerCadena(DateTime valor, string formato)
        {
            return valor.ToString(formato);
        }
    }
    
    public static class Correo
    {
        //Expresión regular que define el formato del correo: identificación, arroba, dominio y país.
        public const string FORMATO_CORREO = @"^([\w-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([\w-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$";
        
        public static bool EsValido(string valor)
        {
            Regex rx = new Regex(FORMATO_CORREO);
            
            return rx.IsMatch(valor);
        }
    }
    
    public static class Telefono
    {
        //Expresión regular que define el formato del teléfono: 9 cifras seguidas.
        public const string FORMATO_TELEFONO = @"^\d{9}$";
        
        public static bool EsValido(string valor)
        {
            Regex rx = new Regex(FORMATO_TELEFONO);
            
            return rx.IsMatch(valor);
        }
    }
    
    public static class CodigoPostal
    {
        //Expresión regular que define el formato del código postal: 5 cifras seguidas con restricción de valores.
        public const string FORMATO_CODIGO_POSTAL = @"^([1-9]{2}|[0-9][1-9]|[1-9][0-9])[0-9]{3}$";
        
        public static bool EsValido(string valor)
        {
            Regex rx = new Regex(FORMATO_CODIGO_POSTAL);
            
            return rx.IsMatch(valor);
        }
    }
}